/*  Copyright © 2017 Zoltán Nagy. All rights reserved.
 *  Created by Zoltán Nagy on 29/01/2017.
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program. If not, see <http://www.gnu.org/licenses/>.
 */
#ifndef __duct_latex_template_h__
#define __duct_latex_template_h__ 1




/*   std includes   */

/*   project headers   */
#include <bits/hep-dist.h>




namespace duct {
  
  template<typename _Point, typename _Tp>
  struct latex_template;

  
  /*   Default latex template for any dimension distributions    */
  template<typename _Point, typename _Tp>
  requires std::is_same<_Point, distpoint::hist1d>::value || std::is_arithmetic<_Point>::value
  struct latex_template<_Point, _Tp>
  {
    /*    print a simple template of latex code to make the pretty plot  */
    static void plot(std::ostream& ltx, const std::string& title, const std::string& tblpath, bool is_normalized = false)
    {
      constexpr char xcol = std::is_same<_Point, distpoint::hist1d>::value ? '1' : '0';
      constexpr int c1 = std::is_same<_Point, distpoint::hist1d>::value ? 3 : 1;
      
      /*    writting the latex template file   */
      ltx<<"\n\\subsubsection*{\\it "<<title<<"}\n\n"
      <<"\\begin{tikzpicture}\n"
      "\\begin{axis}[title = {"<<title<<"},\n"
      "   xlabel={-- x label --}, ylabel={-- y label --},\n"
      "%  xmin=, xmax=,\n"
      "%  ymin=, ymax=\n]\n"
      "\\errorband[red,semithick]{fill=red!30!white, opacity=0.5}{"<<tblpath<<"}{"<<xcol<<"}{"<<c1<<"}{"<<c1+1<<"}\n";
      
      if(is_normalized)
        ltx<<"%\\errorband[blue,semithick]{fill=blue!30!white, opacity=0.5}{"<<tblpath<<"}{"<<xcol<<"}{"<<c1+2<<"}{"<<c1+3<<"}\n";
      
      ltx<<
      "\n\\end{axis}\n"
      "\\end{tikzpicture}\n\n";
    }
  };
  

  
  /*    Default latex template for arrays    */
  template<typename _Point, typename _Tp, std::size_t _Dim>
  requires std::is_same<_Point, distpoint::hist1d>::value || std::is_arithmetic<_Point>::value
  struct latex_template<_Point, std::array<_Tp, _Dim>>
  {
    /*    print a simple template of latex code to make the pretty plot  */
    static void plot(std::ostream& ltx, const std::string& title, const std::string& tblpath, bool is_normalized = false)
    {
      constexpr char xcol = std::is_same<_Point, distpoint::hist1d>::value ? '1' : '0';
      constexpr int c1 = std::is_same<_Point, distpoint::hist1d>::value ? 3 : 1;
      
      /*    writting the latex template file   */
      ltx<<"\n\\subsubsection*{\\it "<<title<<"}\n\n"
      <<"\\begin{tikzpicture}\n"
      <<"\\begin{axis}[title = {"<<title<<"},\n"
      <<"   xlabel={-- x label --}, ylabel={-- y label --},\n"
      <<"%  xmin=, xmax=,\n"
      <<"%  ymin=, ymax=\n]\n"
      <<"\\errorband[red,semithick]{fill=red!30!white, opacity=0.5}{"<<tblpath<<"}{"<<xcol<<"}{"<<c1<<"}{"<<c1+1<<"}\n\n";
      
      for(std::size_t i = 1; i < (is_normalized ? 2*_Dim : _Dim); ++i) {
        ltx<<"%\\errorband[blue,semithick]{fill=blue!30!white, opacity=0.5}{"<<tblpath<<"}{"<<xcol<<"}{"<<c1+2*i<<"}{"<<c1+1+2*i<<"}\n";
      }
      
      ltx
      <<"\\end{axis}\n"
      <<"\\end{tikzpicture}\n\n";
    }
  };
  
  
  
  
  /*   Default print for zero dimension distributions    */
  template<typename _Tp>
  struct latex_template<distpoint::nulld, _Tp>
  {
    /*    latex template */
    static void plot(std::ostream& ltx, const std::string& title, const std::string& tblpath)
    {
      /*    writting the latex template file   */
      ltx<<"\n\\subsubsection*{\\it "<<title<<"}\n\n"
      <<"\\begin{equation}\n"
      <<"\\sigma = \\left[\n"
      <<"\\input{"<<tblpath<<"}\n"
      <<"\\right]\n"
      <<"\\end{equation}\n\n";
    }
    
  };
  

}   //   namespace duct

#endif
