/*  Copyright © 2015 Zoltán Nagy. All rights reserved.
 *  Created by Zoltán Nagy on 16/12/15.
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program. If not, see <http://www.gnu.org/licenses/>.
 */
#ifndef __math_expint_h__
#define __math_expint_h__ 1


/*   std includes   */

/*   project headers   */


namespace duct {
  
  namespace __details {
    
    /* implementation for E1, allowing for scaling by exp(x) */
    double expint_E1_impl(const double x, bool scale);
    
  } /*   namespace __details   */
  
  
  /*  real part of E1 function     */
  inline double expint_E1(double x) {
    return __details::expint_E1_impl(x, false);
  }
  
  
  /*  real part of E1 function scaled by exp(x)    */
  inline double expint_E1_scaled(double x) {
    return __details::expint_E1_impl(x, true);
  }
  
  /*    Ei(x)  integral exponential function   */
  inline double expint_Ei(double x) {
    return -expint_E1(-x);
  }
  
  /*    Ei(x)  integral exponential function scaled by exp(x)  */
  inline double expint_Ei_scaled(double x) {
    return -expint_E1_scaled(-x);
  }
  

}

#endif