/*  Copyright © 2015 Zoltán Nagy. All rights reserved.
 *  Created by Zoltán Nagy on 10/12/15.
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program. If not, see <http://www.gnu.org/licenses/>.
 */
#ifndef __math_harmonic_numbers_h__
#define __math_harmonic_numbers_h__ 1


/*   std includes   */

/*   project headers   */


namespace duct {
  namespace __details {

    
    template<std::size_t N>
    struct harmonic_num_helper
    {
      static constexpr double value = 1.0/N + harmonic_num_helper<N-1>::value;
    };
    
    template<>
    struct harmonic_num_helper<1>
    {
      static constexpr double value = 1.0;
    };
    
    template<>
    struct harmonic_num_helper<0>
    {
      static constexpr double value = 0.0;
    };
    
  }   /*   namespace __details   */
  

  namespace meta {
    
    /*   Harmonic numbers
     *    Use it carefully!!! It is computed in compiler time
     *    by recursive template the recurance can break down for large N.
     */
    template<std::size_t N>
    double harmonic_number = __details::harmonic_num_helper<N>::value;
  }
  
  
  
  
}

#endif