/*  Copyright © 2015 Zoltán Nagy. All rights reserved.
 *  Created by Zoltán Nagy on 10/12/15.
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program. If not, see <http://www.gnu.org/licenses/>.
 */
#ifndef __math_pow_h__
#define __math_pow_h__ 1


/*   std includes   */

/*   project headers   */


namespace duct {
  
  namespace __details {
    
    /*  Helper class for computing the power functions   */
    template<std::size_t N, class Tp, bool is_odd = N%2>
    struct pow_helper;
    
    /*   Specialization for odd powers   */
    template<std::size_t N, class Tp>
    struct pow_helper<N, Tp, true>
    {
      static constexpr Tp pow(const Tp& x) {
        return x*pow_helper<2,Tp>::pow(pow_helper<(N-1)/2, Tp>::pow(x));
      }
    };
    
    
    /*   specialization for even powers    */
    template<std::size_t N, class Tp>
    struct pow_helper<N, Tp, false>
    {
      static constexpr Tp pow(const Tp& x) {
        return pow_helper<2,Tp>::pow(pow_helper<N/2, Tp>::pow(x));
      }
    };
    
    
    /*   Specializations for trivial cases    */
    template<class Tp>
    struct pow_helper<2, Tp, false>
    {
      static constexpr Tp pow(const Tp& x) {
        return x*x;
      }
    };
    
    template<class Tp>
    struct pow_helper<1, Tp, true>
    {
      static constexpr Tp pow(const Tp& x) {
        return x;
      }
    };
    
    template<class Tp>
    struct pow_helper<0, Tp, false>
    {
      static constexpr Tp pow(const Tp&) {
        return 1;
      }
    };
  
  }   /*   namespace __details   */
  
  /*  We put the static version of math function on namespace meta.
   *  This way we can implement their runtime version too.
   */
  namespace meta {
    
    /*   compiler time power function    */
    template<std::size_t N, class Tp>
    constexpr Tp pow(const Tp& x) {
      return __details::pow_helper<N,Tp>::pow(x);
    }
    
  }

  
  /*   simple sqr function   */
  template<class Tp>
  constexpr Tp sqr(const Tp& x) {
    return x*x;
  }
  
  
}

#endif